local mod = ...


--[[

	Register a new behavior for my Authority workers
	Goal is to have this be identical to normal worker behavior, but when transporting
	instead of carrying a box, just walking.

	This is both to remove boxes for noble transporters but also to work around a bug where a transporting
	noble loses the top of their costume when carrying the box, then doesn't get it back.

	I lifted this code from the Example Mod 2 and added the transport behavior.


	
	CONCLUSION:
	Seems to have no effect on the transport animation, pending an answer from Maxime if I am missing something



]]



mod:register({
	Id = "WORKSTATION_AUTHORITY",
	DataType = "AGENT_WORKSTATION_SETUP",
	LocationTypeList = {
		"PICKUP",
	},
	CharacterSetup = {
		DataType = "CHARACTER_SETUP",
		WalkAnimation = "WALKING",
		IdleAnimation = "WALKING"
	}
})




mod:registerBehaviorTreeNode({
    Name = "BEHAVIOR_TREE_NODE_AUTHORITY",
    VariableList = {
		Var1 = "BEHAVIOR_TREE_DATA_AGENT",
		Var2 = "BEHAVIOR_TREE_DATA_WAIT",
		Var3 = "BEHAVIOR_TREE_DATA_LOCATION",
		Var4 = "BEHAVIOR_TREE_DATA_RESOURCE_TRANSPORT",
		Var5 = "BEHAVIOR_TREE_DATA_LOOP"
    },
    Init = function(self, instance)
		mod:log("INIT!")
    end,
    Update = function(self, level, instance)
		mod:log("UPDATE!")
		return self:updateFunc(level, instance)
    end,
    Finish = function(self, instance)
		mod:log("FINISH!")
    end,
	updateFunc = function(self, level, instance)
		mod:log("UPDATE FUNC!")
		mod:log("Agent data: " .. tostring(self.Var1))
		mod:log(tostring(self.Var1.Agent))
		mod:log("Wait data: " .. tostring(self.Var2))
		mod:log("Location data: " .. tostring(self.Var3))
		mod:log("Resource transport data: " .. tostring(self.Var4))
		mod:log("Loop data: " .. tostring(self.Var5))
		return BEHAVIOR_TREE_NODE_RESULT.TRUE
	end,
})

mod:registerBehaviorTree({
	Id = "BEHAVIOR_TREE_AUTHORITY",
	VariableList = {
		{
			Name = "AgentData",
			DataType = "BEHAVIOR_TREE_DATA_AGENT",
			IsPublic = true,
			InitialValue = {} -- Optional if empty, leaves the default values (if any)
		},
		{
			Name = "DoJobTimer",
			DataType = "BEHAVIOR_TREE_DATA_WAIT",
			IsPublic = false,
			InitialValue = {
				TimeToWait = 0,
				Animation = AGENT_ANIMATION.GATHER,
				SetIdleAfterWait = false,
			}
		},
		{
			Name = "WorkplacePosition",
			DataType = "BEHAVIOR_TREE_DATA_LOCATION",
			IsPublic = false,
			InitialValue = {
				CanNavigateOnGround = true,
				CanNavigateOnWater = false,
				IsSetOrientationOnDestination = true,
			}
		},
		{
			Name = "TransportData",
			DataType = "BEHAVIOR_TREE_DATA_RESOURCE_TRANSPORT",
			IsPublic = false,
			TransportSetup = "WORKSTATION_AUTHORITY"
		},
		{
			Name = "WorkLoop",
			DataType = "BEHAVIOR_TREE_DATA_LOOP",
			IsPublic = true,
			InitialValue = {
				LoopCount = 10,
				Duration = 0,
				IsInfinite = false,
				IsDuration = false,
			}
		},
	},
	Root = {
		Name = "WorkingGlobalSequencer",
		Type = "SEQUENCER",
		Children = {
			{
				Name = "NeedFilledSelector",
				Type = "SELECTOR",
				Children = {
					{
						Name = "NeedFilledInverter",
						Type = "INVERTER",
						Child = {
							Name = "IsMyNeedFilled",
							Type = "IS_WORKPLACE_NEED_RESOURCE",
							AgentData = "AgentData",
						}
					},
					{
						Name = "FetchResourceSequencer",
						Type = "SEQUENCER",
						Children = {
							{
								Name = "FindResourceForWorkplace",
								Type = "FIND_RESOURCE_FOR_WORKPLACE",
								AgentData = "AgentData",
								TransportData = "TransportData",
							},
							{
								Name = "Transport",
								Type = "BEHAVIOR_TRANSPORT",
								AgentData = "AgentData",
								TransportData = "TransportData",
							},
						},
					},
				},
			},
			{
				Name = "CUSTOM_NODE_INSTANCE",
				Type = "BEHAVIOR_TREE_NODE_AUTHORITY",
				Var1 = "AgentData",
				Var2 = "DoJobTimer",
				Var3 = "WorkplacePosition",
				Var4 = "TransportData",
				Var5 = "WorkLoop",
			},
			{
				Name = "WorkLoopRepeater",
				Type = "REPEAT",
				RepeatData = "WorkLoop",
				Child = {
					Name = "WorkLoopSequencer",
					Type = "SEQUENCER",
					Children = {
						{
							Name = "SetupWork",
							Type = "SETUP_WORK",
							AgentData = "AgentData",
							TimeToWait = "DoJobTimer",
							WorkPosition = "WorkplacePosition",
						},
						{
							Name = "GoToResource",
							Type = "GO_TO",
							AgentData = "AgentData",
							Destination = "WorkplacePosition",
							BuildingPathType = "",
							AnimationData = "",
							AnimationSpeedMultiplier = "",
						},
						{
							Name = "ProduceResource",
							Type = "PRODUCE_RESOURCE",
							AgentData = "AgentData",
							TimeToWait = "DoJobTimer",
						},
						{
							Name = "GettingXp",
							Type = "GIVE_JOB_XP",
							AgentData = "AgentData",
							ShouldReceiveXp = "",
						}
					}
				}
			}
		}
	}
})